@@ -1,5 +1,5 @@
 /* dvdcpy - A DVD backup tool
- * Copyright (C) 2004-2006 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include <fcntl.h>
 #include <stdio.h>
+#include <getopt.h>
 #include <stdarg.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -143,11 +144,18 @@ dvd_file_size (dvd_reader_t *reader, unsigned int vts, dvd_read_domain_t domain)
 static void
 usage (const char *name)
 {
-  fprintf (stderr, "usage: %s [-o output] [-t title]* device\n", name);
-  fprintf (stderr, "  device    - Specify the DVD device\n");
-  fprintf (stderr, "  -o output - Specify the output directory\n");
-  fprintf (stderr, "  -t title  - Select the title to copy\n");
-  fprintf (stderr, "  -m        - Copy the DVD menu\n");
+  fprintf (stdout, "Usage:\n");
+  fprintf (stdout, "  %s [OPTION...] <DVD DEVICE>\n", name);
+  fprintf (stdout, "\n");
+  fprintf (stdout, "Help Options:\n");
+  fprintf (stdout, "  -h, --help              Show help options\n");
+  fprintf (stdout, "\n");
+  fprintf (stdout, "Application Options:\n");
+  fprintf (stdout, "  -o, --output output     Specify the output directory\n");
+  fprintf (stdout, "  -t, --title title       Select the title to copy. This option may be\n");
+  fprintf (stdout, "                          specified multiple times (default: all titles)\n");
+  fprintf (stdout, "  -m, --menu              Copy the DVD menu\n");
+  fprintf (stdout, "\n");
 }
 
 /*
@@ -262,7 +270,7 @@ get_space_left (const char *path)
 }
 
 static ssize_t
-dvd_copy_blocks (dvd_file_t *file, unsigned int offset, size_t size, int fd)
+dvd_copy_blocks (dvd_file_t *file, size_t offset, size_t size, int fd)
 {
   unsigned char buffer[512 * DVD_VIDEO_LB_LEN];
   unsigned int start;
@@ -309,13 +317,11 @@ dvd_get_vts_size (dvd_reader_t *reader, unsigned int vts)
     return -1;
   fullsize = size;
 
-  if ((size = dvd_file_size (reader, vts, DVD_READ_INFO_BACKUP_FILE)) < 0)
-    return -1;
-  fullsize += size;
+  if ((size = dvd_file_size (reader, vts, DVD_READ_INFO_BACKUP_FILE)) > 0)
+    fullsize += size;
 
-  if ((size = dvd_file_size (reader, vts, DVD_READ_MENU_VOBS)) < 0)
-    return -1;
-  fullsize += size;
+  if ((size = dvd_file_size (reader, vts, DVD_READ_MENU_VOBS)) > 0)
+    fullsize += size;
 
   if (vts > 0)
   {
@@ -336,13 +342,11 @@ dvd_get_vmg_size (dvd_reader_t *reader)
     return -1;
   fullsize = size;
 
-  if ((size = dvd_file_size (reader, 0, DVD_READ_INFO_BACKUP_FILE)) < 0)
-    return -1;
-  fullsize += size;
+  if ((size = dvd_file_size (reader, 0, DVD_READ_INFO_BACKUP_FILE)) > 0)
+    fullsize += size;
 
-  if ((size = dvd_file_size (reader, 0, DVD_READ_MENU_VOBS)) < 0)
-    return -1;
-  fullsize += size;
+  if ((size = dvd_file_size (reader, 0, DVD_READ_MENU_VOBS)) > 0)
+    fullsize += size;
 
   return fullsize;
 }
@@ -412,7 +416,7 @@ dvd_copy_ifo (dvd_reader_t *reader, unsigned int vts, const char *output)
   file = DVDOpenFile (reader, vts, DVD_READ_INFO_FILE);
   if (!file)
   {
-    fprintf (stderr, "Errot: Cannot open IFO for VTS %u\n", vts);
+    fprintf (stderr, "Error: Cannot open IFO for VTS %u\n", vts);
     close (fd);
     return -1;
   }
@@ -470,7 +474,7 @@ dvd_copy_bup (dvd_reader_t *reader, unsigned int vts, const char *output)
   file = DVDOpenFile (reader, vts, DVD_READ_INFO_BACKUP_FILE);
   if (!file)
   {
-    fprintf (stderr, "Errot: Cannot open BUP for VTS %u\n", vts);
+    fprintf (stderr, "Error: Cannot open BUP for VTS %u\n", vts);
     close (fd);
     return -1;
   }
@@ -493,7 +497,7 @@ static ssize_t
 dvd_copy_menu (dvd_reader_t *reader, unsigned int vts, const char *output)
 {
   char filename[FILENAME_MAX];
-  unsigned int size, copied;
+  ssize_t size, copied;
   struct stat buf;
   int fd;
 
@@ -528,7 +532,7 @@ dvd_copy_menu (dvd_reader_t *reader, unsigned int vts, const char *output)
   file = DVDOpenFile (reader, vts, DVD_READ_MENU_VOBS);
   if (!file)
   {
-    fprintf (stderr, "Errot: Cannot open menu for VTS %u\n", vts);
+    fprintf (stderr, "Error: Cannot open menu for VTS %u\n", vts);
     close (fd);
     return -1;
   }
@@ -552,7 +556,7 @@ dvd_copy_vob (dvd_reader_t *reader, unsigned int vts, const char *output)
 {
   char filename[FILENAME_MAX];
   struct stat buf;
-  uint32_t size;
+  ssize_t size;
   size_t copied;
   int vob, fd;
 
@@ -561,14 +565,14 @@ dvd_copy_vob (dvd_reader_t *reader, unsigned int vts, const char *output)
 
   if (DVDFileStat (reader, vts, DVD_READ_TITLE_VOBS, &dvd_stat) < 0)
   {
-    fprintf (stderr, "Errot: Cannot stat VOB for VTS %u\n", vts);
+    fprintf (stderr, "Error: Cannot stat VOB for VTS %u\n", vts);
     return -1;
   }
 
   dvd_file = DVDOpenFile (reader, vts, DVD_READ_TITLE_VOBS);
   if (!dvd_file)
   {
-    fprintf (stderr, "Errot: Cannot open VOB for VTS %u\n", vts);
+    fprintf (stderr, "Error: Cannot open VOB for VTS %u\n", vts);
     return -1;
   }
 
@@ -632,13 +636,11 @@ dvd_copy_vmg (dvd_reader_t *reader, const char *output)
     return -1;
   fullsize = size;
 
-  if ((size = dvd_copy_bup (reader, 0, output)) < 0)
-    return -1;
-  fullsize += size;
+  if ((size = dvd_copy_bup (reader, 0, output)) > 0)
+    fullsize += size;
 
-  if ((size = dvd_copy_menu (reader, 0, output)) < 0)
-    return -1;
-  fullsize += size;
+  if ((size = dvd_copy_menu (reader, 0, output)) > 0)
+    fullsize += size;
 
   return fullsize;
 }
@@ -652,13 +654,11 @@ dvd_copy_vts (dvd_reader_t *reader, unsigned int vts, const char *output)
     return -1;
   fullsize = size;
 
-  if ((size = dvd_copy_bup (reader, vts, output)) < 0)
-    return -1;
-  fullsize += size;
+  if ((size = dvd_copy_bup (reader, vts, output)) > 0)
+    fullsize += size;
 
-  if ((size = dvd_copy_menu (reader, vts, output)) < 0)
-    return -1;
-  fullsize += size;
+  if ((size = dvd_copy_menu (reader, vts, output)) > 0)
+    fullsize += size;
 
   if ((size = dvd_copy_vob (reader, vts, output)) < 0)
     return -1;
@@ -693,6 +693,16 @@ dvd_copy (dvd_reader_t *reader, ifo_handle_t *vmg_file, unsigned int vmg, unsign
   return fullsize;
 }
 
+static const char *shortopts = "hmo:t:";
+static const struct option longopts[] =
+{
+  { "help",     no_argument,       NULL, 'h' },
+  { "menu",     no_argument,       NULL, 'm' },
+  { "output",   required_argument, NULL, 'o' },
+  { "title",    required_argument, NULL, 't' },
+  { NULL,       0,                 NULL,  0  }
+};
+
 int
 main (int argc, char *argv[])
 {
@@ -700,8 +710,8 @@ main (int argc, char *argv[])
 
   dvd_reader_t *reader;
   ifo_handle_t *vmg_file;
+  int c, optidx;
   char *output;
-  int c;
 
   if (argc < 2)
   {
@@ -714,7 +724,7 @@ main (int argc, char *argv[])
 
   memset (titles, 0, 100 * sizeof (int));
 
-  while ((c = getopt (argc, argv, "hmo:t:")) != -1)
+  while ((c = getopt_long (argc, argv, shortopts, longopts, &optidx)) != EOF)
   {
     switch (c)
     {
