@@ -1,5 +1,5 @@
 /* OGMDvd - A wrapper library around libdvdread
- * Copyright (C) 2004-2006 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,19 +20,31 @@
 #include "config.h"
 #endif
 
-#include <glib/gi18n-lib.h>
-
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkcellrenderertext.h>
+/**
+ * SECTION:ogmdvd-chapter-list
+ * @title: OGMDvdChapterList
+ * @include: ogmdvd-chapter-list.h
+ * @short_description: A widget that lists the chapters of a DVD title
+ */
 
 #include "ogmdvd-chapter-list.h"
 
+#include <glib/gi18n-lib.h>
+#include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkliststore.h>
+
+enum
+{
+  PROP_0,
+  PROP_TITLE
+};
+
 enum
 {
   COL_CHAPTER,
   COL_LABEL,
   COL_LENGTH,
-  COL_FRAMES,
+  COL_SECONDS,
   COL_LAST
 };
 
@@ -44,7 +56,15 @@ struct _OGMDvdChapterListPriv
   OGMDvdTitle *title;
 };
 
-static void ogmdvd_chapter_list_finalize (GObject *object);
+static void ogmdvd_chapter_list_dispose      (GObject      *object);
+static void ogmdvd_chapter_list_get_property (GObject      *gobject,
+                                              guint        property_id,
+                                              GValue       *value,
+                                              GParamSpec   *pspec);
+static void ogmdvd_chapter_list_set_property (GObject      *gobject,
+                                              guint        property_id,
+                                              const GValue *value,
+                                              GParamSpec   *pspec);
 
 G_DEFINE_TYPE (OGMDvdChapterList, ogmdvd_chapter_list, GTK_TYPE_TREE_VIEW);
 
@@ -55,7 +75,13 @@ ogmdvd_chapter_list_class_init (OGMDvdChapterListClass *klass)
 
   object_class = (GObjectClass *) klass;
 
-  object_class->finalize = ogmdvd_chapter_list_finalize;
+  object_class->dispose = ogmdvd_chapter_list_dispose;
+  object_class->get_property = ogmdvd_chapter_list_get_property;
+  object_class->set_property = ogmdvd_chapter_list_set_property;
+
+  g_object_class_install_property (object_class, PROP_TITLE, 
+        g_param_spec_pointer ("title", "Title property", "The DVD title",
+          G_PARAM_READWRITE));
 
   g_type_class_add_private (klass, sizeof (OGMDvdChapterListPriv));
 }
@@ -72,7 +98,6 @@ ogmdvd_chapter_list_label_edited (OGMDvdChapterList *list, gchar *path, gchar *t
   gtk_list_store_set (GTK_LIST_STORE (model), &iter, COL_LABEL, text, -1);
 }
 
-
 static void
 ogmdvd_chapter_list_init (OGMDvdChapterList *list)
 {
@@ -98,6 +123,20 @@ ogmdvd_chapter_list_init (OGMDvdChapterList *list)
   g_signal_connect_swapped (renderer, "edited", G_CALLBACK (ogmdvd_chapter_list_label_edited), list);
 }
 
+static void
+ogmdvd_chapter_list_dispose (GObject *object)
+{
+  OGMDvdChapterList *list;
+
+  list = OGMDVD_CHAPTER_LIST (object);
+
+  if (list->priv->title)
+    ogmdvd_title_unref (list->priv->title);
+  list->priv->title = NULL;
+
+  (*G_OBJECT_CLASS (ogmdvd_chapter_list_parent_class)->dispose) (object);
+}
+/*
 static void
 ogmdvd_chapter_list_finalize (GObject *object)
 {
@@ -110,7 +149,42 @@ ogmdvd_chapter_list_finalize (GObject *object)
 
   (*G_OBJECT_CLASS (ogmdvd_chapter_list_parent_class)->finalize) (object);
 }
+*/
+static void
+ogmdvd_chapter_list_get_property (GObject *gobject, guint property_id, GValue *value, GParamSpec *pspec)
+{
+  switch (property_id) 
+  {
+    case PROP_TITLE:
+      g_value_set_pointer (value, OGMDVD_CHAPTER_LIST (gobject)->priv->title);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
+      break;
+  }
+}
 
+static void
+ogmdvd_chapter_list_set_property (GObject *gobject, guint property_id, const GValue *value, GParamSpec *pspec)
+{
+  switch (property_id) 
+  {
+    case PROP_TITLE:
+      ogmdvd_chapter_list_set_title (OGMDVD_CHAPTER_LIST (gobject), g_value_get_pointer (value));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
+      break;
+  }
+}
+
+/**
+ * ogmdvd_chapter_list_new:
+ *
+ * Creates a new empty #OGMDvdChapterList.
+ *
+ * Returns: The new #OGMDvdChapterList
+ */
 GtkWidget *
 ogmdvd_chapter_list_new (void)
 {
@@ -120,12 +194,19 @@ ogmdvd_chapter_list_new (void)
   list = g_object_new (OGMDVD_TYPE_CHAPTER_LIST, NULL);
 
   store = gtk_list_store_new (COL_LAST, 
-      G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_ULONG);
+      G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_DOUBLE);
   gtk_tree_view_set_model (GTK_TREE_VIEW (list), GTK_TREE_MODEL (store));
+  g_object_unref (store);
 
   return GTK_WIDGET (list);
 }
 
+/**
+ * ogmdvd_chapter_list_clear:
+ * @list: An #OGMDvdChapterList
+ *
+ * Removes all entries of the #OGMDvdChapterList.
+ */
 void
 ogmdvd_chapter_list_clear (OGMDvdChapterList *list)
 {
@@ -141,6 +222,13 @@ ogmdvd_chapter_list_clear (OGMDvdChapterList *list)
   list->priv->title = NULL;
 }
 
+/**
+ * ogmdvd_chapter_list_set_title:
+ * @list: An #OGMDvdChapterList
+ * @title: An #OGMDvdTitle
+ *
+ * Adds to the list the chapters of the given #OGMDvdTitle.
+ */
 void
 ogmdvd_chapter_list_set_title (OGMDvdChapterList *list, OGMDvdTitle *title)
 {
@@ -154,7 +242,7 @@ ogmdvd_chapter_list_set_title (OGMDvdChapterList *list, OGMDvdTitle *title)
     OGMDvdTime time_;
 
     gint chap, nchap;
-    glong frames;
+    gdouble seconds;
     gchar *str;
 
     ogmdvd_title_ref (title);
@@ -174,16 +262,74 @@ ogmdvd_chapter_list_set_title (OGMDvdChapterList *list, OGMDvdTitle *title)
       gtk_list_store_set (GTK_LIST_STORE (model), &iter, COL_CHAPTER, chap + 1, COL_LABEL, str, -1);
       g_free (str);
 
-      if ((frames = ogmdvd_title_get_chapters_length (title, chap, chap, &time_)) > 0)
+      if ((seconds = ogmdvd_title_get_chapters_length (title, chap, chap, &time_)) > 0)
       {
         str = g_strdup_printf ("%02d:%02d:%02d", time_.hour, time_.min, time_.sec);
-        gtk_list_store_set (GTK_LIST_STORE (model), &iter, COL_LENGTH, str, COL_FRAMES, frames, -1);
+        gtk_list_store_set (GTK_LIST_STORE (model), &iter, COL_LENGTH, str, /*COL_SECONDS, seconds,*/ -1);
         g_free (str);
       }
     }
   }
 }
 
+/**
+ * ogmdvd_chapter_list_get_label:
+ * @list: An #OGMDvdChapterList
+ * @chapter: A chapter number
+ *
+ * Returns the label of the given chapter.
+ *
+ * Returns: The chapter's label
+ */
+gchar *
+ogmdvd_chapter_list_get_label (OGMDvdChapterList *list, guint chapter)
+{
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  gchar *label;
+
+  g_return_val_if_fail (OGMDVD_IS_CHAPTER_LIST (list), NULL);
+
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
+
+  if (!gtk_tree_model_iter_nth_child (model, &iter, NULL, chapter))
+    return NULL;
+
+  gtk_tree_model_get (model, &iter, COL_LABEL, &label, -1);
+
+  return label;
+}
+
+/**
+ * ogmdvd_chapter_list_set_label:
+ * @list: An #OGMDvdChapterList
+ * @chapter: A chapter number
+ * @label: The label of the chapter
+ *
+ * Sets the label of the given chapter.
+ */
+void
+ogmdvd_chapter_list_set_label (OGMDvdChapterList *list, guint chapter, const gchar *label)
+{
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+
+  g_return_if_fail (OGMDVD_IS_CHAPTER_LIST (list));
+
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
+
+  if (gtk_tree_model_iter_nth_child (model, &iter, NULL, chapter))
+    gtk_list_store_set (GTK_LIST_STORE (model), &iter, COL_LABEL, label, -1);
+}
+
+/**
+ * ogmdvd_chapter_list_get_title:
+ * @list: An #OGMDvdChapterList
+ *
+ * Returns the #OGMDvdTitle which was passed to ogmdvd_chapter_list_set_title().
+ *
+ * Returns: An #OGMDvdTitle
+ */
 OGMDvdTitle *
 ogmdvd_chapter_list_get_title (OGMDvdChapterList *list)
 {
@@ -192,33 +338,3 @@ ogmdvd_chapter_list_get_title (OGMDvdChapterList *list)
   return list->priv->title;
 }
 
-gboolean
-ogmdvd_chapter_list_get_chapter (OGMDvdChapterList *list, guint chapter, gchar **label, gulong *frames)
-{
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-
-  g_return_val_if_fail (OGMDVD_IS_CHAPTER_LIST (list), FALSE);
-
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
-  if (gtk_tree_model_iter_nth_child (model, &iter, NULL, chapter))
-  {
-    gchar *l;
-    gulong f;
-
-    gtk_tree_model_get (model, &iter, COL_LABEL, &l, COL_FRAMES, &f, -1);
-
-    if (label)
-      *label = l;
-    else
-      g_free (l);
-
-    if (frames)
-      *frames = f;
-
-    return TRUE;
-  }
-
-  return FALSE;
-}
-
