@@ -1,5 +1,5 @@
 /* OGMDvd - A wrapper library around libdvdread
- * Copyright (C) 2004-2006 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,12 +19,10 @@
 #ifndef __OGMDVD_GTK_H__
 #define __OGMDVD_GTK_H__
 
-#include <ogmdvd-audio-chooser.h>
 #include <ogmdvd-chapter-list.h>
-#include <ogmdvd-drive-chooser.h>
-#include <ogmdvd-drive-dialog.h>
-#include <ogmdvd-subtitle-chooser.h>
-#include <ogmdvd-title-chooser.h>
+#include <ogmdvd-drive-chooser-widget.h>
+#include <ogmdvd-drive-chooser-dialog.h>
+#include <ogmdvd-title-chooser-widget.h>
 
 #endif /* __OGMDVD_GTK_H__ */
 
