@@ -1,5 +1,5 @@
 /* OGMDvd - A wrapper library around libdvdread
- * Copyright (C) 2004-2006 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,35 +19,36 @@
 #ifndef __OGMDVD_TITLE_CHOOSER_H__
 #define __OGMDVD_TITLE_CHOOSER_H__
 
+#include <gtk/gtkwidget.h>
 #include <ogmdvd.h>
 
-#include <gtk/gtkcombobox.h>
-
 G_BEGIN_DECLS
 
 #define OGMDVD_TYPE_TITLE_CHOOSER            (ogmdvd_title_chooser_get_type ())
 #define OGMDVD_TITLE_CHOOSER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), OGMDVD_TYPE_TITLE_CHOOSER, OGMDvdTitleChooser))
-#define OGMDVD_TITLE_CHOOSER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), OGMDVD_TYPE_TITLE_CHOOSER, OGMDvdTitleChooserClass))
-#define OGMDVD_IS_TITLE_CHOOSER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, OGMDVD_TYPE_TITLE_CHOOSER))
-#define OGMDVD_IS_TITLE_CHOOSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), OGMDVD_TYPE_TITLE_CHOOSER))
+#define OGMDVD_IS_TITLE_CHOOSER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), OGMDVD_TYPE_TITLE_CHOOSER))
+#define OGMDVD_TITLE_CHOOSER_GET_IFACE(inst) (G_TYPE_INSTANCE_GET_INTERFACE ((inst), OGMDVD_TYPE_TITLE_CHOOSER, OGMDvdTitleChooserIface))
 
 typedef struct _OGMDvdTitleChooser      OGMDvdTitleChooser;
-typedef struct _OGMDvdTitleChooserClass OGMDvdTitleChooserClass;
-typedef struct _OGMDvdTitleChooserPriv  OGMDvdTitleChooserPriv;
+typedef struct _OGMDvdTitleChooserIface OGMDvdTitleChooserIface;
 
-struct _OGMDvdTitleChooser
+struct _OGMDvdTitleChooserIface
 {
-  GtkComboBox widget;
-  OGMDvdTitleChooserPriv *priv;
+  GTypeInterface base_iface;
+
+  /*
+   * Methods
+   */
+
+  void          (* set_disc)   (OGMDvdTitleChooser *chooser,
+                                OGMDvdDisc         *disc);
+  OGMDvdDisc *  (* get_disc)   (OGMDvdTitleChooser *chooser);
+  OGMDvdTitle * (* get_active) (OGMDvdTitleChooser *chooser);
 };
 
-struct _OGMDvdTitleChooserClass
-{
-  GtkComboBoxClass parent_class;
-};
 
-GType         ogmdvd_title_chooser_get_type   (void);
-GtkWidget *   ogmdvd_title_chooser_new        (void);
+GType         ogmdvd_title_chooser_get_type   (void) G_GNUC_CONST;
+
 void          ogmdvd_title_chooser_set_disc   (OGMDvdTitleChooser *chooser,
                                                OGMDvdDisc         *disc);
 OGMDvdDisc *  ogmdvd_title_chooser_get_disc   (OGMDvdTitleChooser *chooser);
