@@ -1,5 +1,5 @@
 /* OGMJob - A library to spawn processes
- * Copyright (C) 2004-2006 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,6 +16,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
+/**
+ * SECTION:ogmjob-container
+ * @title: OGMJobContainer
+ * @include: ogmjob-container.h
+ * @short_description: Base class for spawns which contain other spawns
+ */
+
 #include "ogmjob-container.h"
 
 #define OGMJOB_CONTAINER_GET_PRIVATE(o) \
@@ -28,8 +35,10 @@ enum
   LAST_SIGNAL
 };
 
-static void ogmjob_container_finalize    (GObject         *object);
+static void ogmjob_container_dispose     (GObject         *object);
 static void ogmjob_container_cancel      (OGMJobSpawn     *spawn);
+static void ogmjob_container_suspend     (OGMJobSpawn     *spawn);
+static void ogmjob_container_resume      (OGMJobSpawn     *spawn);
 static void ogmjob_container_real_add    (OGMJobContainer *container, 
                                           OGMJobSpawn     *child);
 static void ogmjob_container_real_remove (OGMJobContainer *container,
@@ -48,24 +57,38 @@ ogmjob_container_class_init (OGMJobContainerClass *klass)
   gobject_class = G_OBJECT_CLASS (klass);
   spawn_class = OGMJOB_SPAWN_CLASS (klass);
 
-  gobject_class->finalize = ogmjob_container_finalize;
+  gobject_class->dispose = ogmjob_container_dispose;
 
   spawn_class->cancel = ogmjob_container_cancel;
+  spawn_class->suspend = ogmjob_container_suspend;
+  spawn_class->resume = ogmjob_container_resume;
 
   klass->add = ogmjob_container_real_add;
   klass->remove = ogmjob_container_real_remove;
 
-  signals[ADD] = g_signal_new ("add", 
-      G_TYPE_FROM_CLASS (gobject_class), G_SIGNAL_RUN_FIRST, 
-      G_STRUCT_OFFSET (OGMJobContainerClass, add), 
-      NULL, NULL, g_cclosure_marshal_VOID__OBJECT, 
-      G_TYPE_NONE, 1, OGMJOB_TYPE_SPAWN);
+  /**
+   * OGMJobContainer::add
+   * @container: the container that received the signal
+   * @child: the child to be added
+   *
+   * Emitted each time a child is added to a container.
+   */
+  signals[ADD] = g_signal_new ("add", G_TYPE_FROM_CLASS (gobject_class),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS, 
+      G_STRUCT_OFFSET (OGMJobContainerClass, add), NULL, NULL,
+      g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, OGMJOB_TYPE_SPAWN);
 
-  signals[REMOVE] = g_signal_new ("remove", 
-      G_TYPE_FROM_CLASS (gobject_class), G_SIGNAL_RUN_FIRST, 
-      G_STRUCT_OFFSET (OGMJobContainerClass, remove), 
-      NULL, NULL, g_cclosure_marshal_VOID__OBJECT, 
-      G_TYPE_NONE, 1, OGMJOB_TYPE_SPAWN);
+  /**
+   * OGMJobContainer::remove
+   * @container: the container that received the signal
+   * @child: the child to be removed
+   *
+   * Emitted each time a child is removed from a container.
+   */
+  signals[REMOVE] = g_signal_new ("remove", G_TYPE_FROM_CLASS (gobject_class),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS, 
+      G_STRUCT_OFFSET (OGMJobContainerClass, remove), NULL, NULL,
+      g_cclosure_marshal_VOID__OBJECT, G_TYPE_NONE, 1, OGMJOB_TYPE_SPAWN);
 }
 
 static void
@@ -74,12 +97,12 @@ ogmjob_container_init (OGMJobContainer *container)
 }
 
 static void
-ogmjob_container_finalize (GObject *gobject)
+ogmjob_container_dispose (GObject *gobject)
 {
   ogmjob_container_foreach (OGMJOB_CONTAINER (gobject), 
       (OGMJobCallback) g_object_unref, NULL);
 
-  G_OBJECT_CLASS (ogmjob_container_parent_class)->finalize (gobject);
+  G_OBJECT_CLASS (ogmjob_container_parent_class)->dispose (gobject);
 }
 
 static void
@@ -89,18 +112,39 @@ ogmjob_container_cancel (OGMJobSpawn *spawn)
       (OGMJobCallback) ogmjob_spawn_cancel, NULL);
 }
 
-void
+static void
+ogmjob_container_suspend (OGMJobSpawn *spawn)
+{
+  ogmjob_container_foreach (OGMJOB_CONTAINER (spawn), 
+      (OGMJobCallback) ogmjob_spawn_suspend, NULL);
+}
+
+static void
+ogmjob_container_resume (OGMJobSpawn *spawn)
+{
+  ogmjob_container_foreach (OGMJOB_CONTAINER (spawn), 
+      (OGMJobCallback) ogmjob_spawn_resume, NULL);
+}
+
+static void
 ogmjob_container_real_add (OGMJobContainer *container, OGMJobSpawn *spawn)
 {
   g_warning ("Not implemented\n");
 }
 
-void
+static void
 ogmjob_container_real_remove (OGMJobContainer *container, OGMJobSpawn *spawn)
 {
   g_warning ("Not implemented\n");
 }
 
+/**
+ * ogmjob_container_add:
+ * @container: An #OGMJobContainer
+ * @spawn: An #OGMJobSpawn
+ *
+ * Adds @spawn to @container.
+ */
 void
 ogmjob_container_add (OGMJobContainer *container, OGMJobSpawn *spawn)
 {
@@ -116,6 +160,13 @@ ogmjob_container_add (OGMJobContainer *container, OGMJobSpawn *spawn)
   }
 }
 
+/**
+ * ogmjob_container_remove:
+ * @container: An #OGMJobContainer
+ * @spawn: An #OGMJobSpawn
+ *
+ * Removes @spawn from @container.
+ */
 void
 ogmjob_container_remove (OGMJobContainer *container, OGMJobSpawn *spawn)
 {
@@ -127,6 +178,14 @@ ogmjob_container_remove (OGMJobContainer *container, OGMJobSpawn *spawn)
   g_signal_emit (container, signals[REMOVE], 0, spawn);
 }
 
+/**
+ * ogmjob_container_foreach:
+ * @container: An #OGMJobContainer
+ * @callback: A callback
+ * @data: Callback user data
+ *
+ * Invokes @callback on each child of @container.
+ */
 void
 ogmjob_container_foreach (OGMJobContainer *container, OGMJobCallback callback, gpointer data)
 {
