@@ -1,5 +1,5 @@
 /* OGMJob - A library to spawn processes
- * Copyright (C) 2004-2006 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,9 +21,9 @@
 
 #include <ogmjob-bin.h>
 #include <ogmjob-container.h>
-#include <ogmjob-enums.h>
 #include <ogmjob-exec.h>
 #include <ogmjob-list.h>
+#include <ogmjob-log.h>
 #include <ogmjob-pipeline.h>
 #include <ogmjob-queue.h>
 #include <ogmjob-spawn.h>
