@@ -1,5 +1,5 @@
 /* OGMRip - A library for DVD ripping and encoding
- * Copyright (C) 2004-2006 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,6 +16,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
+/**
+ * SECTION:ogmrip-codec
+ * @title: OGMRipCodec
+ * @short_description: Base class for codecs
+ * @include: ogmrip-codec.h
+ */
+
 #include "ogmrip-codec.h"
 
 #include <unistd.h>
@@ -34,9 +41,11 @@ struct _OGMRipCodecPriv
   guint framerate_denominator;
   guint framestep;
 
+  OGMRipEdl *edl;
+
   gboolean do_unlink;
   gboolean dirty;
-  gulong length;
+  gdouble length;
 
   guint start;
   gint end;
@@ -55,6 +64,7 @@ enum
   PROP_FRAMESTEP
 };
 
+static void ogmrip_codec_dispose      (GObject      *gobject);
 static void ogmrip_codec_finalize     (GObject      *gobject);
 static void ogmrip_codec_set_property (GObject      *gobject,
                                        guint        property_id,
@@ -74,6 +84,7 @@ ogmrip_codec_class_init (OGMRipCodecClass *klass)
 
   gobject_class = G_OBJECT_CLASS (klass);
 
+  gobject_class->dispose = ogmrip_codec_dispose;
   gobject_class->finalize = ogmrip_codec_finalize;
   gobject_class->set_property = ogmrip_codec_set_property;
   gobject_class->get_property = ogmrip_codec_get_property;
@@ -87,8 +98,8 @@ ogmrip_codec_class_init (OGMRipCodecClass *klass)
            NULL, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_LENGTH, 
-        g_param_spec_long ("length", "Length property", "Get length", 
-           0, G_MAXLONG, 0, G_PARAM_READABLE));
+        g_param_spec_double ("length", "Length property", "Get length", 
+           0.0, G_MAXDOUBLE, 0.0, G_PARAM_READABLE));
 
   g_object_class_install_property (gobject_class, PROP_START_CHAPTER, 
         g_param_spec_int ("start-chapter", "Start chapter property", "Set start chapter", 
@@ -121,6 +132,24 @@ ogmrip_codec_init (OGMRipCodec *codec)
   codec->priv->end = -1;
 }
 
+static void
+ogmrip_codec_dispose (GObject *gobject)
+{
+  OGMRipCodec *codec;
+
+  codec = OGMRIP_CODEC (gobject);
+
+  if (codec->priv->title)
+    ogmdvd_title_unref (codec->priv->title);
+  codec->priv->title = NULL;
+
+  if (codec->priv->edl)
+    ogmrip_edl_unref (codec->priv->edl);
+  codec->priv->edl = NULL;
+
+  G_OBJECT_CLASS (ogmrip_codec_parent_class)->dispose (gobject);
+}
+
 static void
 ogmrip_codec_finalize (GObject *gobject)
 {
@@ -137,10 +166,6 @@ ogmrip_codec_finalize (GObject *gobject)
     codec->priv->output = NULL;
   }
 
-  if (codec->priv->title)
-    ogmdvd_title_unref (codec->priv->title);
-  codec->priv->title = NULL;
-
   G_OBJECT_CLASS (ogmrip_codec_parent_class)->finalize (gobject);
 }
 
@@ -196,7 +221,7 @@ ogmrip_codec_get_property (GObject *gobject, guint property_id, GValue *value, G
       g_value_set_static_string (value, codec->priv->output);
       break;
     case PROP_LENGTH: 
-      g_value_set_long (value, ogmrip_codec_get_length (codec, NULL));
+      g_value_set_double (value, ogmrip_codec_get_length (codec, NULL));
       break;
     case PROP_START_CHAPTER: 
       g_value_set_int (value, codec->priv->start);
@@ -219,6 +244,13 @@ ogmrip_codec_get_property (GObject *gobject, guint property_id, GValue *value, G
   }
 }
 
+/**
+ * ogmrip_codec_set_output:
+ * @codec: an #OGMRipCodec
+ * @output: the name of the output file
+ *
+ * Sets the name of the output file.
+ */
 void
 ogmrip_codec_set_output (OGMRipCodec *codec, const gchar *output)
 {
@@ -228,6 +260,14 @@ ogmrip_codec_set_output (OGMRipCodec *codec, const gchar *output)
   codec->priv->output = g_strdup (output);
 }
 
+/**
+ * ogmrip_codec_get_output:
+ * @codec: an #OGMRipCodec
+ *
+ * Gets the name of the output file.
+ *
+ * Returns: the filename, or NULL
+ */
 G_CONST_RETURN gchar *
 ogmrip_codec_get_output (OGMRipCodec *codec)
 {
@@ -236,6 +276,13 @@ ogmrip_codec_get_output (OGMRipCodec *codec)
   return codec->priv->output;
 }
 
+/**
+ * ogmrip_codec_set_input:
+ * @codec: an #OGMRipCodec
+ * @title: an #OGMDvdTitle
+ *
+ * Sets the input DVD title.
+ */
 void
 ogmrip_codec_set_input (OGMRipCodec *codec, OGMDvdTitle *title)
 {
@@ -258,6 +305,14 @@ ogmrip_codec_set_input (OGMRipCodec *codec, OGMDvdTitle *title)
   codec->priv->end = -1;
 }
 
+/**
+ * ogmrip_codec_get_input:
+ * @codec: an #OGMRipCodec
+ *
+ * Gets the input DVD title.
+ *
+ * Returns: an #OGMDvdTitle, or NULL
+ */
 OGMDvdTitle *
 ogmrip_codec_get_input (OGMRipCodec *codec)
 {
@@ -266,6 +321,51 @@ ogmrip_codec_get_input (OGMRipCodec *codec)
   return codec->priv->title;
 }
 
+/**
+ * ogmrip_codec_set_edl:
+ * @codec: an #OGMRipCodec
+ * @edl: an #OGMRipEdl
+ *
+ * Sets an edit decision list file.
+ */
+void
+ogmrip_codec_set_edl (OGMRipCodec *codec, OGMRipEdl *edl)
+{
+  g_return_if_fail (OGMRIP_IS_CODEC (codec));
+
+  if (edl)
+    ogmrip_edl_ref (edl);
+  if (codec->priv->edl)
+    ogmrip_edl_unref (codec->priv->edl);
+
+  codec->priv->edl = edl;
+  codec->priv->dirty = TRUE;
+}
+
+/**
+ * ogmrip_codec_get_edl:
+ * @codec: an #OGMRipCodec
+ *
+ * Gets the edit decision list if any.
+ *
+ * Returns: an #OGMRipEdl, or NULL
+ */
+OGMRipEdl *
+ogmrip_codec_get_edl (OGMRipCodec *codec)
+{
+  g_return_val_if_fail (OGMRIP_IS_CODEC (codec), NULL);
+
+  return codec->priv->edl;
+}
+
+/**
+ * ogmrip_codec_set_chapters:
+ * @codec: an #OGMRipCodec
+ * @start: the start chapter
+ * @end: the end chapter
+ *
+ * Sets which chapters to start and end at.
+ */
 void
 ogmrip_codec_set_chapters (OGMRipCodec *codec, guint start, gint end)
 {
@@ -287,6 +387,14 @@ ogmrip_codec_set_chapters (OGMRipCodec *codec, guint start, gint end)
   }
 }
 
+/**
+ * ogmrip_codec_get_chapters:
+ * @codec: an #OGMRipCodec
+ * @start: a pointer to set the start chapter
+ * @end: a pointer to set the end chapter
+ *
+ * Gets which chapters to start and end at.
+ */
 void
 ogmrip_codec_get_chapters (OGMRipCodec *codec, guint *start, guint *end)
 {
@@ -302,38 +410,115 @@ ogmrip_codec_get_chapters (OGMRipCodec *codec, guint *start, guint *end)
     *end = codec->priv->end;
 }
 
+/**
+ * ogmrip_codec_get_frames:
+ * @codec: an #OGMRipCodec
+ *
+ * Returns the length of the encoding in frames.
+ *
+ * Returns: the length in frames, or -1
+ */
 glong
-ogmrip_codec_get_length (OGMRipCodec *codec, OGMDvdTime *time_)
+ogmrip_codec_get_frames (OGMRipCodec *codec)
 {
+  gdouble framerate;
+
   g_return_val_if_fail (OGMRIP_IS_CODEC (codec), -1);
 
   if (!codec->priv->title)
     return -1;
 
   if (codec->priv->dirty)
-  {
-    guint numerator, denominator;
+    ogmrip_codec_get_length (codec, NULL);
 
+  framerate = codec->priv->framerate_numerator / (gdouble) codec->priv->framerate_denominator;
+
+  return codec->priv->length * framerate;
+}
+
+static void
+ogmrip_codec_foreach_edl_element (OGMRipEdlAction action, gdouble start, gdouble end, gdouble data[])
+{
+  if (action == OGMRIP_EDL_ACTION_SKIP && start <= data[0])
+  {
+    if (end > data[0])
+      data[1] -= data[0] - start;
+    else
+      data[1] -= end - start;
+  }
+}
+
+/**
+ * ogmrip_codec_get_length:
+ * @codec: an #OGMRipCodec
+ * @length: a pointer to store an #OGMDvdTime, or NULL
+ *
+ * Returns the length of the encoding in seconds. If @length is not NULL, the
+ * data structure will be filled with the length in hours, minutes seconds and
+ * frames.
+ *
+ * Returns: the length in seconds, or -1.0
+ */
+gdouble
+ogmrip_codec_get_length (OGMRipCodec *codec, OGMDvdTime *length)
+{
+  g_return_val_if_fail (OGMRIP_IS_CODEC (codec), -1.0);
+
+  if (!codec->priv->title)
+    return -1.0;
+
+  if (codec->priv->dirty)
+  {
     if (codec->priv->start == 0 && codec->priv->end == -1)
       codec->priv->length = ogmdvd_title_get_length (codec->priv->title, &codec->priv->time_);
     else
       codec->priv->length = ogmdvd_title_get_chapters_length (codec->priv->title, 
             codec->priv->start, codec->priv->end, &codec->priv->time_);
 
+    /*
+     * TODO EDL can be dirty !!!!
+     */
+
+    if (codec->priv->edl)
+    {
+      gdouble data[2];
+      gulong msecs;
+
+      data[0] = data[1] = codec->priv->length;
+      ogmrip_edl_foreach (codec->priv->edl, (OGMRipEdlFunc) ogmrip_codec_foreach_edl_element, data);
+      codec->priv->length = data[1];
+
+      msecs = data[1] * 1000.0;
+
+      codec->priv->time_.hour = msecs / (60 * 60 * 1000);
+      codec->priv->time_.min = msecs / (60 * 1000) % 60;
+      codec->priv->time_.sec = msecs / 1000 % 60;
+      codec->priv->time_.frames = msecs % 1000;
+    }
+/*
     ogmdvd_title_get_framerate (codec->priv->title, &numerator, &denominator);
     if (numerator != codec->priv->framerate_numerator || denominator != codec->priv->framerate_denominator)
       codec->priv->length *= ((denominator * codec->priv->framerate_numerator) / 
           (gdouble) (numerator * codec->priv->framerate_denominator));
-
+*/
     codec->priv->dirty = FALSE;
   }
 
-  if (time_)
-    *time_ = codec->priv->time_;
+  if (length)
+    *length = codec->priv->time_;
 
   return codec->priv->length;
 }
 
+/**
+ * ogmrip_codec_set_framerate:
+ * @codec: an #OGMRipCodec
+ * @numerator: the framerate numerator
+ * @denominator: the framerate denominator
+ *
+ * Sets a frames per second (fps) value for the output file, which can be
+ * different from that of the source material.
+ */
 void
 ogmrip_codec_set_framerate (OGMRipCodec *codec, guint numerator, guint denominator)
 {
@@ -344,6 +529,14 @@ ogmrip_codec_set_framerate (OGMRipCodec *codec, guint numerator, guint denominat
   codec->priv->framerate_denominator = denominator;
 }
 
+/**
+ * ogmrip_codec_get_framerate:
+ * @codec: an #OGMRipCodec
+ * @numerator: a pointer to store the framerate numerator
+ * @denominator: a pointer to store the framerate denominator
+ *
+ * Gets the framerate of the output file in the form of a fraction.
+ */
 void
 ogmrip_codec_get_framerate (OGMRipCodec *codec, guint *numerator, guint *denominator)
 {
@@ -353,7 +546,7 @@ ogmrip_codec_get_framerate (OGMRipCodec *codec, guint *numerator, guint *denomin
 
   *numerator = codec->priv->framerate_numerator;
   *denominator = codec->priv->framerate_denominator;
-
+/*
   if (!numerator || !denominator)
   {
     OGMDvdTitle *title;
@@ -361,8 +554,16 @@ ogmrip_codec_get_framerate (OGMRipCodec *codec, guint *numerator, guint *denomin
     title = ogmrip_codec_get_input (OGMRIP_CODEC (codec));
     ogmdvd_title_get_framerate (title, numerator, denominator);
   }
+*/
 }
 
+/**
+ * ogmrip_codec_set_framestep:
+ * @codec: an #OGMRipCodec
+ * @framestep: the framestep
+ *
+ * Skips @framestep frames after every frame.
+ */
 void
 ogmrip_codec_set_framestep (OGMRipCodec *codec, guint framestep)
 {
@@ -371,6 +572,14 @@ ogmrip_codec_set_framestep (OGMRipCodec *codec, guint framestep)
   codec->priv->framestep = MAX (framestep, 1);
 }
 
+/**
+ * ogmrip_codec_get_framestep:
+ * @codec: an #OGMRipCodec
+ *
+ * Gets the number of frames to skip after every frame.
+ *
+ * Returns: the framestep, or -1
+ */
 gint
 ogmrip_codec_get_framestep (OGMRipCodec *codec)
 {
@@ -379,6 +588,14 @@ ogmrip_codec_get_framestep (OGMRipCodec *codec)
   return codec->priv->framestep;
 }
 
+/**
+ * ogmrip_codec_set_unlink_on_unref:
+ * @codec: an #OGMRipCodec
+ * @do_unlink: %TRUE to unlink the output file
+ *
+ * Whether to unlink the output file on the final unref of the #OGMRipCodec
+ * data structure.
+ */
 void
 ogmrip_codec_set_unlink_on_unref (OGMRipCodec *codec, gboolean do_unlink)
 {
@@ -387,6 +604,15 @@ ogmrip_codec_set_unlink_on_unref (OGMRipCodec *codec, gboolean do_unlink)
   codec->priv->do_unlink = do_unlink;
 }
 
+/**
+ * ogmrip_codec_get_unlink_on_unref:
+ * @codec: an #OGMRipCodec
+ *
+ * Returns whether the output file will be unlinked on the final unref of
+ * @codec.
+ *
+ * Returns: a boolean
+ */
 gboolean
 ogmrip_codec_get_unlink_on_unref (OGMRipCodec *codec)
 {
