@@ -1,5 +1,5 @@
 /* OGMRip - A library for DVD ripping and encoding
- * Copyright (C) 2004-2006 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,14 +16,22 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
+/**
+ * SECTION:ogmrip-container
+ * @title: OGMRipContainer
+ * @short_description: Base class for containers
+ * @include: ogmrip-container.h
+ */
+
 #include "ogmrip-container.h"
-#include "ogmrip-backend.h"
-#include "ogmjob-enums.h"
+#include "ogmrip-plugin.h"
 #include "ogmjob-exec.h"
 
 #include <unistd.h>
 #include <glib/gstdio.h>
 
+#define DEFAULT_OVERHEAD 6
+
 #define OGMRIP_CONTAINER_GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_CONTAINER, OGMRipContainerPriv))
 
@@ -35,11 +43,13 @@ struct _OGMRipContainerPriv
 
   guint tsize;
   guint tnumber;
-  guint overhead;
+  guint start_delay;
+
+  GSList *subp;
+  GSList *audio;
+  GSList *chapters;
+  GSList *files;
 
-  GList *subp;
-  GList *audio;
-  GList *chapters;
   OGMRipVideo *video;
 };
 
@@ -58,9 +68,11 @@ enum
   PROP_FOURCC,
   PROP_TSIZE,
   PROP_TNUMBER,
-  PROP_OVERHEAD
+  PROP_OVERHEAD,
+  PROP_START_DELAY
 };
 
+static void ogmrip_container_dispose      (GObject      *gobject);
 static void ogmrip_container_finalize     (GObject      *gobject);
 static void ogmrip_container_set_property (GObject      *gobject,
                                            guint        property_id,
@@ -82,6 +94,7 @@ ogmrip_container_class_init (OGMRipContainerClass *klass)
   gobject_class = G_OBJECT_CLASS (klass);
   spawn_class = OGMJOB_SPAWN_CLASS (klass);
 
+  gobject_class->dispose  = ogmrip_container_dispose;
   gobject_class->finalize = ogmrip_container_finalize;
   gobject_class->set_property = ogmrip_container_set_property;
   gobject_class->get_property = ogmrip_container_get_property;
@@ -107,8 +120,12 @@ ogmrip_container_class_init (OGMRipContainerClass *klass)
            0, G_MAXUINT, 1, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_OVERHEAD, 
-        g_param_spec_uint ("overhead", "Overhead property", "Set overhead", 
-           0, G_MAXUINT, 6, G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
+        g_param_spec_uint ("overhead", "Overhead property", "Get overhead", 
+           0, G_MAXUINT, 6, G_PARAM_READABLE));
+
+  g_object_class_install_property (gobject_class, PROP_START_DELAY, 
+        g_param_spec_uint ("start-delay", "Start delay property", "Set start delay", 
+           0, G_MAXINT, 0, G_PARAM_READWRITE));
 
   g_type_class_add_private (klass, sizeof (OGMRipContainerPriv));
 }
@@ -117,7 +134,6 @@ static void
 ogmrip_container_init (OGMRipContainer *container)
 {
   container->priv = OGMRIP_CONTAINER_GET_PRIVATE (container);
-  container->priv->overhead = 6;
   container->priv->tnumber = 1;
   container->priv->tsize = 0;
 }
@@ -132,7 +148,7 @@ ogmrip_stream_free (OGMRipContainerChild *child)
 }
 
 static void
-ogmrip_container_finalize (GObject *gobject)
+ogmrip_container_dispose (GObject *gobject)
 {
   OGMRipContainer *container;
 
@@ -142,18 +158,30 @@ ogmrip_container_finalize (GObject *gobject)
     g_object_unref (container->priv->video);
   container->priv->video = NULL;
 
-  g_list_foreach (container->priv->audio, (GFunc) ogmrip_stream_free, NULL);
-  g_list_free (container->priv->audio);
+  g_slist_foreach (container->priv->audio, (GFunc) ogmrip_stream_free, NULL);
+  g_slist_free (container->priv->audio);
   container->priv->audio = NULL;
 
-  g_list_foreach (container->priv->chapters, (GFunc) ogmrip_stream_free, NULL);
-  g_list_free (container->priv->chapters);
+  g_slist_foreach (container->priv->chapters, (GFunc) ogmrip_stream_free, NULL);
+  g_slist_free (container->priv->chapters);
   container->priv->chapters = NULL;
 
-  g_list_foreach (container->priv->subp, (GFunc) ogmrip_stream_free, NULL);
-  g_list_free (container->priv->subp);
+  g_slist_foreach (container->priv->subp, (GFunc) ogmrip_stream_free, NULL);
+  g_slist_free (container->priv->subp);
   container->priv->subp = NULL;
 
+  g_slist_foreach (container->priv->files, (GFunc) ogmrip_file_unref, NULL);
+  g_slist_free (container->priv->files);
+  container->priv->files = NULL;
+}
+
+static void
+ogmrip_container_finalize (GObject *gobject)
+{
+  OGMRipContainer *container;
+
+  container = OGMRIP_CONTAINER (gobject);
+
   g_free (container->priv->label);
   container->priv->label = NULL;
 
@@ -190,8 +218,8 @@ ogmrip_container_set_property (GObject *gobject, guint property_id, const GValue
     case PROP_TNUMBER: 
       container->priv->tnumber = g_value_get_uint (value);
       break;
-    case PROP_OVERHEAD: 
-      container->priv->overhead = g_value_get_uint (value);
+    case PROP_START_DELAY: 
+      container->priv->start_delay = g_value_get_uint (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
@@ -203,6 +231,7 @@ static void
 ogmrip_container_get_property (GObject *gobject, guint property_id, GValue *value, GParamSpec *pspec)
 {
   OGMRipContainer *container;
+  gint overhead;
 
   container = OGMRIP_CONTAINER (gobject);
 
@@ -224,7 +253,11 @@ ogmrip_container_get_property (GObject *gobject, guint property_id, GValue *valu
       g_value_set_uint (value, container->priv->tnumber);
       break;
     case PROP_OVERHEAD:
-      g_value_set_uint (value, container->priv->overhead);
+      overhead = ogmrip_container_get_overhead (container);
+      g_value_set_uint (value, overhead > 0 ? overhead : 6);
+      break;
+    case PROP_START_DELAY:
+      g_value_set_uint (value, container->priv->start_delay);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
@@ -232,16 +265,16 @@ ogmrip_container_get_property (GObject *gobject, guint property_id, GValue *valu
   }
 }
 
-static GList *
-ogmrip_container_copy_list (GList *list)
+static GSList *
+ogmrip_container_copy_list (GSList *list)
 {
   OGMRipContainerChild *child;
-  GList *new_list = NULL;
+  GSList *new_list = NULL;
 
   while (list)
   {
     child = list->data;
-    new_list = g_list_append (new_list, child->codec);
+    new_list = g_slist_append (new_list, child->codec);
     list = list->next;
   }
 
@@ -249,7 +282,7 @@ ogmrip_container_copy_list (GList *list)
 }
 
 static void
-ogmrip_container_foreach (OGMRipContainer *container, GList *list, OGMRipContainerFunc func, gpointer data)
+ogmrip_container_foreach_codec (OGMRipContainer *container, GSList *list, OGMRipContainerCodecFunc func, gpointer data)
 {
   OGMRipContainerChild *child;
 
@@ -261,6 +294,14 @@ ogmrip_container_foreach (OGMRipContainer *container, GList *list, OGMRipContain
   }
 }
 
+/**
+ * ogmrip_container_get_output:
+ * @container: An #OGMRipContainer
+ *
+ * Gets the name of the output file.
+ *
+ * Returns: The filename, or NULL
+ */
 G_CONST_RETURN gchar *
 ogmrip_container_get_output (OGMRipContainer *container)
 {
@@ -269,6 +310,13 @@ ogmrip_container_get_output (OGMRipContainer *container)
   return container->priv->output;
 }
 
+/**
+ * ogmrip_container_set_output:
+ * @container: an #OGMRipContainer
+ * @output: the name of the output file
+ *
+ * Sets the name of the output file.
+ */
 void
 ogmrip_container_set_output (OGMRipContainer *container, const gchar *output)
 {
@@ -279,6 +327,14 @@ ogmrip_container_set_output (OGMRipContainer *container, const gchar *output)
   container->priv->output = g_strdup (output);
 }
 
+/**
+ * ogmrip_container_get_label:
+ * @container: An #OGMRipContainer
+ *
+ * Gets the label of the rip.
+ *
+ * Returns: The label, or NULL
+ */
 G_CONST_RETURN gchar *
 ogmrip_container_get_label (OGMRipContainer *container)
 {
@@ -287,6 +343,13 @@ ogmrip_container_get_label (OGMRipContainer *container)
   return container->priv->label;
 }
 
+/**
+ * ogmrip_container_set_label:
+ * @container: An #OGMRipContainer
+ * @label: the label
+ *
+ * Sets the label of the rip.
+ */
 void
 ogmrip_container_set_label (OGMRipContainer *container, const gchar *label)
 {
@@ -296,6 +359,14 @@ ogmrip_container_set_label (OGMRipContainer *container, const gchar *label)
   container->priv->label = label ? g_strdup (label) : NULL;
 }
 
+/**
+ * ogmrip_container_get_fourcc:
+ * @container: An #OGMRipContainer
+ *
+ * Gets the FourCC of the rip.
+ *
+ * Returns: The FourCC, or NULL
+ */
 G_CONST_RETURN gchar *
 ogmrip_container_get_fourcc (OGMRipContainer *container)
 {
@@ -304,6 +375,13 @@ ogmrip_container_get_fourcc (OGMRipContainer *container)
   return container->priv->fourcc;
 }
 
+/**
+ * ogmrip_container_set_fourcc:
+ * @container: An #OGMRipContainer
+ * @fourcc: the FourCC
+ *
+ * Sets the FourCC of the rip.
+ */
 void
 ogmrip_container_set_fourcc (OGMRipContainer *container, const gchar *fourcc)
 {
@@ -323,6 +401,67 @@ ogmrip_container_set_fourcc (OGMRipContainer *container, const gchar *fourcc)
   }
 }
 
+/**
+ * ogmrip_container_get_start_delay:
+ * @container: An #OGMRipContainer
+ *
+ * Gets the start delay of the audio tracks.
+ *
+ * Returns: The start delay, or -1
+ */
+gint
+ogmrip_container_get_start_delay (OGMRipContainer *container)
+{
+  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);
+
+  return container->priv->start_delay;
+}
+
+/**
+ * ogmrip_container_set_start_delay:
+ * @container: An #OGMRipContainer
+ * @start_delay: the start delay
+ *
+ * Sets the start delay of the audio tracks
+ */
+void
+ogmrip_container_set_start_delay (OGMRipContainer *container, guint start_delay)
+{
+  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
+
+  container->priv->start_delay = start_delay;
+}
+
+/**
+ * ogmrip_container_get_overhead:
+ * @container: An #OGMRipContainer
+ *
+ * Gets the overhead of the container.
+ *
+ * Returns: The overhead, or -1
+ */
+gint
+ogmrip_container_get_overhead (OGMRipContainer *container)
+{
+  OGMRipContainerClass *klass;
+
+  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);
+
+  klass = OGMRIP_CONTAINER_GET_CLASS (container);
+  if (klass->get_overhead)
+    return (* klass->get_overhead) (container);
+
+  return DEFAULT_OVERHEAD;
+}
+
+/**
+ * ogmrip_container_get_video:
+ * @container: An #OGMRipContainer
+ *
+ * Gets the video codec of the rip.
+ *
+ * Returns: An #OGMRipVideo, or NULL
+ */
 OGMRipVideo *
 ogmrip_container_get_video (OGMRipContainer *container)
 {
@@ -331,6 +470,13 @@ ogmrip_container_get_video (OGMRipContainer *container)
   return container->priv->video;
 }
 
+/**
+ * ogmrip_container_set_video:
+ * @container: An #OGMRipContainer
+ * @video: An #OGMRipVideo
+ *
+ * Sets the video codec of the rip.
+ */
 void
 ogmrip_container_set_video (OGMRipContainer *container, OGMRipVideo *video)
 {
@@ -341,8 +487,20 @@ ogmrip_container_set_video (OGMRipContainer *container, OGMRipVideo *video)
   if (container->priv->video)
     g_object_unref (container->priv->video);
   container->priv->video = video;
+
+  if (!ogmrip_plugin_get_container_bframes (G_TYPE_FROM_INSTANCE (container)))
+    ogmrip_video_set_max_b_frames (video, 0);
 }
 
+/**
+ * ogmrip_container_add_audio:
+ * @container: An #OGMRipContainer
+ * @audio: An #OGMRipAudio
+ * @demuxer: The demuxer to be used
+ * @language: The language of the stream
+ *
+ * Adds an audio codec to the rip.
+ */
 void
 ogmrip_container_add_audio (OGMRipContainer *container, OGMRipAudio *audio, OGMRipAudioDemuxer demuxer, gint language)
 {
@@ -359,10 +517,18 @@ ogmrip_container_add_audio (OGMRipContainer *container, OGMRipAudio *audio, OGMR
   child->language = language;
   child->demuxer = demuxer;
 
-  container->priv->audio = g_list_append (container->priv->audio, child);
+  container->priv->audio = g_slist_append (container->priv->audio, child);
 }
 
-GList *
+/**
+ * ogmrip_container_get_audio:
+ * @container: An #OGMRipContainer
+ *
+ * Gets a list of the audio codecs of the rip.
+ *
+ * Returns: A #GSList, or NULL
+ */
+GSList *
 ogmrip_container_get_audio (OGMRipContainer *container)
 {
   g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);
@@ -370,18 +536,27 @@ ogmrip_container_get_audio (OGMRipContainer *container)
   return ogmrip_container_copy_list (container->priv->audio);
 }
 
+/**
+ * ogmrip_container_get_nth_audio:
+ * @container: an #OGMRipContainer
+ * @n: The index of the audio codec
+ *
+ * Gets the audio codec at the given position.
+ *
+ * Returns: An #OGMRipAudio, or NULL
+ */
 OGMRipAudio *
 ogmrip_container_get_nth_audio (OGMRipContainer *container, gint n)
 {
   OGMRipContainerChild *child;
-  GList *link;
+  GSList *link;
 
   g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);
 
   if (n < 0)
-    link = g_list_last (container->priv->audio);
+    link = g_slist_last (container->priv->audio);
   else
-    link = g_list_nth (container->priv->audio, n);
+    link = g_slist_nth (container->priv->audio, n);
 
   if (!link)
     return NULL;
@@ -391,23 +566,48 @@ ogmrip_container_get_nth_audio (OGMRipContainer *container, gint n)
   return OGMRIP_AUDIO (child->codec);
 }
 
+/**
+ * ogmrip_container_get_n_audio:
+ * @container: an #OGMRipContainer
+ *
+ * Gets the number of audio codecs.
+ *
+ * Returns: the number of audio codecs
+ */
 gint
 ogmrip_container_get_n_audio (OGMRipContainer *container)
 {
   g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);
 
-  return g_list_length (container->priv->audio);
+  return g_slist_length (container->priv->audio);
 }
 
+/**
+ * ogmrip_container_foreach_audio:
+ * @container: an #OGMRipContainer
+ * @func: The function to call with each audio codec
+ * @data: User data to pass to the function
+ *
+ * Calls a function for each audio codec
+ */
 void
-ogmrip_container_foreach_audio (OGMRipContainer *container, OGMRipContainerFunc func, gpointer data)
+ogmrip_container_foreach_audio (OGMRipContainer *container, OGMRipContainerCodecFunc func, gpointer data)
 {
   g_return_if_fail (OGMRIP_IS_CONTAINER (container));
   g_return_if_fail (func != NULL);
 
-  ogmrip_container_foreach (container, container->priv->audio, func, data);
+  ogmrip_container_foreach_codec (container, container->priv->audio, func, data);
 }
 
+/**
+ * ogmrip_container_add_subp:
+ * @container: An #OGMRipContainer
+ * @subp: An #OGMRipSubp
+ * @demuxer: The demuxer to be used
+ * @language: The language of the stream
+ *
+ * Adds a subtitle codec to the rip.
+ */
 void
 ogmrip_container_add_subp (OGMRipContainer *container, OGMRipSubp *subp, OGMRipSubpDemuxer demuxer, gint language)
 {
@@ -424,10 +624,18 @@ ogmrip_container_add_subp (OGMRipContainer *container, OGMRipSubp *subp, OGMRipS
   child->language = language;
   child->demuxer = demuxer;
 
-  container->priv->subp = g_list_append (container->priv->subp, child);
+  container->priv->subp = g_slist_append (container->priv->subp, child);
 }
 
-GList *
+/**
+ * ogmrip_container_get_subp:
+ * @container: An #OGMRipContainer
+ *
+ * Gets a list of the subtitle codecs of the rip.
+ *
+ * Returns: A #GSList, or NULL
+ */
+GSList *
 ogmrip_container_get_subp (OGMRipContainer *container)
 {
   g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);
@@ -435,18 +643,27 @@ ogmrip_container_get_subp (OGMRipContainer *container)
   return ogmrip_container_copy_list (container->priv->subp);
 }
 
+/**
+ * ogmrip_container_get_nth_subp:
+ * @container: an #OGMRipContainer
+ * @n: The index of the subtitle codec
+ *
+ * Gets the subtitle codec at the given position.
+ *
+ * Returns: An #OGMRipSubp, or NULL
+ */
 OGMRipSubp *
 ogmrip_container_get_nth_subp (OGMRipContainer *container, gint n)
 {
   OGMRipContainerChild *child;
-  GList *link;
+  GSList *link;
 
   g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);
 
   if (n < 0)
-    link = g_list_last (container->priv->subp);
+    link = g_slist_last (container->priv->subp);
   else
-    link = g_list_nth (container->priv->subp, n);
+    link = g_slist_nth (container->priv->subp, n);
 
   if (!link)
     return NULL;
@@ -456,25 +673,49 @@ ogmrip_container_get_nth_subp (OGMRipContainer *container, gint n)
   return OGMRIP_SUBP (child->codec);
 }
 
+/**
+ * ogmrip_container_get_n_subp:
+ * @container: an #OGMRipContainer
+ *
+ * Gets the number of subtitle codecs.
+ *
+ * Returns: the number of subtitle codecs
+ */
 gint
 ogmrip_container_get_n_subp (OGMRipContainer *container)
 {
   g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);
 
-  return g_list_length (container->priv->subp);
+  return g_slist_length (container->priv->subp);
 }
 
+/**
+ * ogmrip_container_foreach_subp:
+ * @container: An #OGMRipContainer
+ * @func: The function to call with each subtitle codec
+ * @data: User data to pass to the function
+ *
+ * Calls a function for each subtitle codec
+ */
 void
-ogmrip_container_foreach_subp (OGMRipContainer *container, OGMRipContainerFunc func, gpointer data)
+ogmrip_container_foreach_subp (OGMRipContainer *container, OGMRipContainerCodecFunc func, gpointer data)
 {
   g_return_if_fail (OGMRIP_IS_CONTAINER (container));
   g_return_if_fail (func != NULL);
 
-  ogmrip_container_foreach (container, container->priv->subp, func, data);
+  ogmrip_container_foreach_codec (container, container->priv->subp, func, data);
 }
 
+/**
+ * ogmrip_container_add_chapters:
+ * @container: An #OGMRipContainer
+ * @chapters: An #OGMRipChapters
+ * @language: The language of the chapters
+ *
+ * Adds a chapters codec to the rip.
+ */
 void
-ogmrip_container_add_chapters (OGMRipContainer *container, OGMRipChapters *chapters)
+ogmrip_container_add_chapters (OGMRipContainer *container, OGMRipChapters *chapters, gint language)
 {
   OGMRipContainerChild *child;
 
@@ -485,11 +726,20 @@ ogmrip_container_add_chapters (OGMRipContainer *container, OGMRipChapters *chapt
 
   g_object_ref (chapters);
   child->codec = OGMRIP_CODEC (chapters);
+  child->language = language;
 
-  container->priv->chapters = g_list_append (container->priv->chapters, child);
+  container->priv->chapters = g_slist_append (container->priv->chapters, child);
 }
 
-GList *
+/**
+ * ogmrip_container_get_chapters:
+ * @container: An #OGMRipContainer
+ *
+ * Gets a list of the chapters codecs of the rip.
+ *
+ * Returns: A #GSList, or NULL
+ */
+GSList *
 ogmrip_container_get_chapters (OGMRipContainer *container)
 {
   g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);
@@ -497,18 +747,27 @@ ogmrip_container_get_chapters (OGMRipContainer *container)
   return ogmrip_container_copy_list (container->priv->chapters);
 }
 
+/**
+ * ogmrip_container_get_nth_chapters:
+ * @container: an #OGMRipContainer
+ * @n: The index of the chapters codec
+ *
+ * Gets the chapters codec at the given position.
+ *
+ * Returns: An #OGMRipChapters, or NULL
+ */
 OGMRipChapters *
 ogmrip_container_get_nth_chapters (OGMRipContainer *container, gint n)
 {
   OGMRipContainerChild *child;
-  GList *link;
+  GSList *link;
 
   g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);
 
   if (n < 0)
-    link = g_list_last (container->priv->chapters);
+    link = g_slist_last (container->priv->chapters);
   else
-    link = g_list_nth (container->priv->chapters, n);
+    link = g_slist_nth (container->priv->chapters, n);
 
   if (!link)
     return NULL;
@@ -518,23 +777,139 @@ ogmrip_container_get_nth_chapters (OGMRipContainer *container, gint n)
   return OGMRIP_CHAPTERS (child->codec);
 }
 
+/**
+ * ogmrip_container_get_n_chapters:
+ * @container: an #OGMRipContainer
+ *
+ * Gets the number of chapters codecs.
+ *
+ * Returns: the number of chapters codecs
+ */
 gint
 ogmrip_container_get_n_chapters (OGMRipContainer *container)
 {
   g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);
 
-  return g_list_length (container->priv->chapters);
+  return g_slist_length (container->priv->chapters);
 }
 
+/**
+ * ogmrip_container_foreach_chapters:
+ * @container: An #OGMRipContainer
+ * @func: The function to call with each chapters codec
+ * @data: User data to pass to the function
+ *
+ * Calls a function for each chapters codec
+ */
 void
-ogmrip_container_foreach_chapters (OGMRipContainer *container, OGMRipContainerFunc func, gpointer data)
+ogmrip_container_foreach_chapters (OGMRipContainer *container, OGMRipContainerCodecFunc func, gpointer data)
 {
   g_return_if_fail (OGMRIP_IS_CONTAINER (container));
   g_return_if_fail (func != NULL);
 
-  ogmrip_container_foreach (container, container->priv->chapters, func, data);
+  ogmrip_container_foreach_codec (container, container->priv->chapters, func, data);
 }
 
+/**
+ * ogmrip_container_add_file:
+ * @container: An #OGMRipContainer
+ * @file: An #OOGMRipFile
+ *
+ * Adds a file to the rip.
+ */
+void
+ogmrip_container_add_file (OGMRipContainer *container, OGMRipFile *file)
+{
+  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
+  g_return_if_fail (file != NULL);
+
+  ogmrip_file_ref (file);
+
+  container->priv->files = g_slist_append (container->priv->files, file);
+}
+
+/**
+ * ogmrip_container_get_files:
+ * @container: An #OGMRipContainer
+ *
+ * Gets a list of the files of the rip.
+ *
+ * Returns: A #GSList, or NULL
+ */
+GSList *
+ogmrip_container_get_files (OGMRipContainer *container)
+{
+  return g_slist_copy (container->priv->files);
+}
+
+/**
+ * ogmrip_container_get_nth_file:
+ * @container: an #OGMRipContainer
+ * @n: The index of the file
+ *
+ * Gets the file at the given position.
+ *
+ * Returns: An #OGMRipFile, or NULL
+ */
+OGMRipFile *
+ogmrip_container_get_nth_file (OGMRipContainer *container, gint n)
+{
+  void *file;
+
+  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), NULL);
+
+  if (n < 0)
+    file = g_slist_last (container->priv->chapters);
+  else
+    file = g_slist_nth (container->priv->chapters, n);
+
+  return file;
+}
+
+/**
+ * ogmrip_container_get_n_files:
+ * @container: an #OGMRipContainer
+ *
+ * Gets the number of files.
+ *
+ * Returns: the number of files
+ */
+gint
+ogmrip_container_get_n_files (OGMRipContainer *container)
+{
+  g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);
+
+  return g_slist_length (container->priv->files);
+}
+
+/**
+ * ogmrip_container_foreach_file:
+ * @container: An #OGMRipContainer
+ * @func: The function to call with each file
+ * @data: User data to pass to the function
+ *
+ * Calls a function for each file
+ */
+void
+ogmrip_container_foreach_file (OGMRipContainer *container, OGMRipContainerFileFunc func, gpointer data)
+{
+  GSList *link;
+
+  g_return_if_fail (OGMRIP_IS_CONTAINER (container));
+  g_return_if_fail (func != NULL);
+  
+  for (link = container->priv->files; link; link = link->next)
+    func (container, OGMRIP_FILE (link->data), data);
+}
+
+/**
+ * ogmrip_container_set_split:
+ * @container: An #OGMRipContainer
+ * @number:  The number of file
+ * @size: The size of each file
+ *
+ * Sets the number of output files and the maximum size of each one.
+ */
 void
 ogmrip_container_set_split (OGMRipContainer *container, guint number, guint size)
 {
@@ -547,6 +922,14 @@ ogmrip_container_set_split (OGMRipContainer *container, guint number, guint size
      container->priv->tsize = size;
 }
 
+/**
+ * ogmrip_container_get_split:
+ * @container: An #OGMRipContainer
+ * @number:  A pointer to store the number of file
+ * @size: A pointer to store the size of each file
+ *
+ * Gets the number of output files and the maximum size of each one.
+ */
 void
 ogmrip_container_get_split (OGMRipContainer *container, guint *number, guint *size)
 {
@@ -562,16 +945,15 @@ ogmrip_container_get_split (OGMRipContainer *container, guint *number, guint *si
 static gint64
 ogmrip_container_get_audio_overhead (OGMRipContainer *container, OGMRipContainerChild *child)
 {
-  glong length;
-  gint samples_per_frame, sample_rate;
-  guint numerator, denominator;
+  gdouble length;
+  gint samples_per_frame, sample_rate, overhead;
 
   length = ogmrip_codec_get_length (child->codec, NULL);
   sample_rate = ogmrip_audio_get_sample_rate (OGMRIP_AUDIO (child->codec));
   samples_per_frame = ogmrip_audio_get_samples_per_frame (OGMRIP_AUDIO (child->codec));
-  ogmrip_codec_get_framerate (OGMRIP_CODEC (child->codec), &numerator, &denominator);
+  overhead = ogmrip_container_get_overhead (container);
 
-  return (gint64) length * sample_rate * denominator * container->priv->overhead / (gdouble) (samples_per_frame * numerator);
+  return (gint64) (length * sample_rate * overhead / (gdouble) samples_per_frame);
 }
 
 static gint64
@@ -580,23 +962,45 @@ ogmrip_container_get_subp_overhead (OGMRipContainer *container, OGMRipContainerC
   return 0;
 }
 
+static gint64
+ogmrip_container_get_file_overhead (OGMRipContainer *container, OGMRipFile *file)
+{
+  glong length;
+  gint samples_per_frame, sample_rate, overhead;
+
+  if (ogmrip_file_get_type (file) == OGMRIP_FILE_TYPE_SUBP)
+    return 0;
+
+  length = ogmrip_audio_file_get_length (OGMRIP_AUDIO_FILE (file));
+  sample_rate = ogmrip_audio_file_get_sample_rate (OGMRIP_AUDIO_FILE (file));
+  samples_per_frame = ogmrip_audio_file_get_samples_per_frame (OGMRIP_AUDIO_FILE (file));
+  overhead = ogmrip_container_get_overhead (container);
+
+  return (gint64) length * sample_rate * overhead / (gdouble) samples_per_frame;
+}
+
+/**
+ * ogmrip_container_get_overhead_size:
+ * @container: An #OGMRipContainer
+ *
+ * Returns the size of the overhead generated by the video, audio and subtitle
+ * stream, the chapters information and the files in bytes.
+ * 
+ * Returns: The overhead size
+ */
 gint64
 ogmrip_container_get_overhead_size (OGMRipContainer *container)
 {
-  GList *link;
-  gint64 overhead;
-  guint numerator, denominator;
-  glong length;
+  GSList *link;
+  gint64 overhead = 0;
+  gdouble length;
 
   g_return_val_if_fail (OGMRIP_IS_CONTAINER (container), -1);
 
-  overhead = 0;
-
   if (container->priv->video)
   {
     length = ogmrip_codec_get_length (OGMRIP_CODEC (container->priv->video), NULL);
-    ogmrip_codec_get_framerate (OGMRIP_CODEC (container->priv->video), &numerator, &denominator);
-    overhead = (gint64) length * numerator * container->priv->overhead / denominator;
+    overhead = (gint64) length * ogmrip_container_get_overhead (container);
   }
 
   for (link = container->priv->audio; link; link = link->next)
@@ -605,11 +1009,14 @@ ogmrip_container_get_overhead_size (OGMRipContainer *container)
   for (link = container->priv->subp; link; link = link->next)
     overhead += ogmrip_container_get_subp_overhead (container, link->data);
 
+  for (link = container->priv->files; link; link = link->next)
+    overhead += ogmrip_container_get_file_overhead (container, link->data);
+
   return overhead;
 }
 
 static gint64
-ogmrip_container_get_size (OGMRipContainer *container, OGMRipContainerChild *child)
+ogmrip_container_child_get_size (OGMRipContainerChild *child)
 {
   struct stat buf;
   const gchar *filename;
@@ -623,20 +1030,32 @@ ogmrip_container_get_size (OGMRipContainer *container, OGMRipContainerChild *chi
   return size;
 }
 
+/**
+ * ogmrip_container_get_nonvideo_size:
+ * @container: An #OGMRipContainer
+ *
+ * Returns the size of the audio and subtitle streams, the chapters information
+ * and the files in bytes.
+ * 
+ * Returns: The nonvideo size
+ */
 gint64
 ogmrip_container_get_nonvideo_size (OGMRipContainer *container)
 {
-  GList *link;
+  GSList *link;
   gint64 nonvideo = 0;
 
   for (link = container->priv->audio; link; link = link->next)
-    nonvideo += ogmrip_container_get_size (container, link->data);
+    nonvideo += ogmrip_container_child_get_size (link->data);
 
   for (link = container->priv->subp; link; link = link->next)
-    nonvideo += ogmrip_container_get_size (container, link->data);
+    nonvideo += ogmrip_container_child_get_size (link->data);
 
   for (link = container->priv->chapters; link; link = link->next)
-    nonvideo += ogmrip_container_get_size (container, link->data);
+    nonvideo += ogmrip_container_child_get_size (link->data);
+
+  for (link = container->priv->files; link; link = link->next)
+    nonvideo += ogmrip_file_get_size (link->data);
 
   return nonvideo;
 }
