@@ -1,5 +1,5 @@
 /* OGMRip - A library for DVD ripping and encoding
- * Copyright (C) 2004-2006 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,6 +32,13 @@ G_BEGIN_DECLS
 #define g_chdir chdir
 #endif
 
+/**
+ * OGMRipFsError:
+ * @OGMRIP_FS_ERROR_NONE: No error
+ * @OGMRIP_FS_ERROR_INTERNAL: Unknown error
+ *
+ * Error codes returned by OGMRipFs functions.
+ */
 typedef enum
 {
   OGMRIP_FS_ERROR_NONE,
@@ -57,6 +64,10 @@ gchar *  ogmrip_fs_mkftemp          (const gchar  *tmpl,
                                      GError       **error);
 gchar *  ogmrip_fs_mkdtemp          (const gchar  *tmpl,
                                      GError       **error);
+gchar *  ogmrip_fs_lntemp           (const gchar  *oldpath,
+                                     const gchar  *newtmpl,
+                                     gboolean     symln,
+                                     GError       **error);
 
 gint     ogmrip_fs_open_tmp         (const gchar  *tmpl,
                                      gchar        **name_used,
@@ -73,6 +84,11 @@ gboolean ogmrip_fs_rename           (const gchar  *old_name,
                                      const gchar  *new_name,
                                      GError       **error);
 
+G_CONST_RETURN
+gchar *  ogmrip_fs_get_extension    (const gchar  *filename);
+gchar *  ogmrip_fs_set_extension    (const gchar  *filename,
+                                     const gchar  *extension);
+
 G_END_DECLS
 
 #endif /* __OGMRIP_FS_H__ */
