@@ -1,5 +1,5 @@
 /* OGMRip - A library for DVD ripping and encoding
- * Copyright (C) 2004-2006 Olivier Rolland <billl@users.sf.net>
+ * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -46,7 +46,7 @@ struct _OGMRipXvidClass
 };
 
 GType         ogmrip_xvid_get_type   (void);
-OGMJobSpawn * ogmrip_xvid_new        (OGMDvdTitle *video,
+OGMJobSpawn * ogmrip_xvid_new        (OGMDvdTitle *title,
                                       const gchar *output);
 void          ogmrip_xvid_set_gmc    (OGMRipXvid  *xvid,
                                       gboolean    gmc);
