@@ -7,15 +7,20 @@
 
 #include "mp_msg.h"
 
+extern int verbosity;
+
 void
 mp_msg (int module, int level, const char *format, ...)
 {
-  va_list va;
+  if (level <= verbosity)
+  {
+    va_list va;
 
-  va_start (va, format);
-  vfprintf (stderr, format, va);
-  va_end (va);
+    va_start (va, format);
+    vfprintf (stderr, format, va);
+    va_end (va);
 
-  fflush (stderr);
+    fflush (stderr);
+  }
 }
 
