@@ -660,9 +660,11 @@ void
 spudec_draw (void *this, draw_func draw_alpha)
 {
   spudec_handle_t *spu = (spudec_handle_t *) this;
-  if (spu->start_pts <= spu->now_pts && spu->now_pts < spu->end_pts
-      && spu->image)
+  if (spu->start_pts <= spu->now_pts && spu->now_pts < spu->end_pts && spu->image)
   {
+    if (spu->forced_subs_only && !spu->is_forced_sub)
+      return;
+
     draw_alpha (spu->start_col, spu->start_row, spu->width, spu->height,
                 spu->image, spu->aimage, spu->stride);
     spu->spu_changed = 0;
