@@ -32,20 +32,17 @@
 #include <math.h>
 
 #include <theora/theora.h>
-#include <vorbis/codec.h>
-#include <vorbis/vorbisenc.h>
 
-const char *optstring = "o:v:V:s:S:f:F:";
+const char *optstring = "ho:b:q:a:f:";
 struct option options[] = 
 {
-  { "output", required_argument, NULL, 'o' },
-  { "video-rate-target", required_argument, NULL, 'V' },
-  { "video-quality", required_argument, NULL, 'v' },
-  { "aspect-numerator", optional_argument, NULL, 's' },
-  { "aspect-denominator", optional_argument, NULL, 'S' },
-  { "framerate-numerator", optional_argument, NULL, 'f' },
-  { "framerate-denominator", optional_argument, NULL, 'F' },
-  { NULL, 0, NULL, 0 }
+  { "help",      no_argument,       NULL, 'h' },
+  { "output",    required_argument, NULL, 'o' },
+  { "bitrate",   required_argument, NULL, 'b' },
+  { "quality",   required_argument, NULL, 'q' },
+  { "aspect",    required_argument, NULL, 'a' },
+  { "framerate", required_argument, NULL, 'f' },
+  { NULL,        0,                 NULL,  0  }
 };
 
 FILE *video = NULL;
@@ -65,34 +62,28 @@ int video_r = -1;
 int video_q = 16;
 
 static void
-usage (void)
+usage (const char *name)
 {
-  fprintf (stderr, 
-      "Usage: theoraenc [options] video_file\n"
-      "Options: \n"
-      "  -o --output <filename.ogg>     file name for encoded output;\n"
-      "                                 If this option is not given, the\n"
-      "                                 compressed data is sent to stdout.\n"
-      "  -V --video-rate-target <n>     bitrate target for Theora video\n"
-      "  -v --video-quality <n>         Theora quality selector from 0 to 10\n"
-      "                                 (0 yields smallest files but lowest\n"
-      "                                 video quality. 10 yields highest\n"
-      "                                 fidelity but large files).\n"
-      "  -s --aspect-numerator <n>      Aspect ratio numerator, default is 0\n"
-      "                                 or extracted from YUV input file\n"
-      "  -S --aspect-denominator <n>    Aspect ratio denominator, default is 0\n"
-      "                                 or extracted from YUV input file\n"
-      "  -f --framerate-numerator <n>   Frame rate numerator, can be extracted\n"
-      "                                 from YUV input file. ex: 30000000\n"
-      "  -F --framerate-denominator <n> Frame rate denominator, can be extracted\n"
-      "                                 from YUV input file. ex: 1000000\n"
-      "                                 The frame rate nominator divided by this\n"
-      "                                 determinates the frame rate in units per tick\n"
-      "theoraenc accepts only YUV4MPEG2 uncompressed video.\n");
-  exit (1);
+  fprintf (stdout, "Usage:\n");
+  fprintf (stdout, "  %s [OPTION...] <VIDEO FILE>\n", name);
+  fprintf (stdout, "\n");
+  fprintf (stdout, "Help Options:\n");
+  fprintf (stdout, "  -h, --help        Show help options\n");
+  fprintf (stdout, "\n");
+  fprintf (stdout, "Application Options:\n");
+  fprintf (stdout, "  -o, --output=<filename>     Use filename for output (default: stdout)\n");
+  fprintf (stdout, "  -b, --bitrate=<bitrate>     Bitrate target for Theora video\n");
+  fprintf (stdout, "  -q, --quality=<quality>     Theora quality selector from 0 to 10 (0 yields\n");
+  fprintf (stdout, "                              smallest files but lowest video quality. 10 yields\n");
+  fprintf (stdout, "                              highest fidelity but large files)\n");
+  fprintf (stdout, "  -a, --aspect=<aspect>       Set the aspect ratio of the input file\n");
+  fprintf (stdout, "  -f, --framerate=<rate>      Set the framerate of the input file\n");
+  fprintf (stdout, "\n");
+  fprintf (stdout, "%s only accepts YUV4MPEG2 uncompressed video.\n", name);
+  fprintf (stdout, "\n");
 }
 
-static void
+static int
 id_file (char *f)
 {
   FILE *test;
@@ -116,7 +107,7 @@ id_file (char *f)
     if (!test)
     {
       fprintf (stderr, "Unable to open file %s.\n", f);
-      exit (1);
+      return -1;
     }
   }
 
@@ -124,7 +115,7 @@ id_file (char *f)
   if (ret < 4)
   {
     fprintf (stderr, "EOF determining file type of file %s.\n", f);
-    exit (1);
+    return -1;
   }
 
   if (!memcmp (buffer, "YUV4", 4))
@@ -158,7 +149,7 @@ id_file (char *f)
          * umm, we already have one 
          */
         fprintf (stderr, "Multiple video files specified on command line.\n");
-        exit (1);
+        return -1;
       }
 
       if (buffer[4] != '2')
@@ -170,7 +161,7 @@ id_file (char *f)
       if (ret < 7)
       {
         fprintf (stderr, "Error parsing YUV4MPEG2 header in file %s.\n", f);
-        exit (1);
+        return -1;
       }
 
       /*
@@ -188,7 +179,7 @@ id_file (char *f)
       if (interlace != 'p')
       {
         fprintf (stderr, "Input video is interlaced; Theora handles only progressive scan\n");
-        exit (1);
+        return -1;
       }
 
       video = test;
@@ -196,16 +187,16 @@ id_file (char *f)
       fprintf (stderr, "File %s is %dx%d %.02f fps YUV12 video.\n", 
           f, frame_x, frame_y, (double) video_hzn / video_hzd);
 
-      return;
+      return 0;
     }
   }
 
   fprintf (stderr, "Input file %s is not a YUV4MPEG2 file.\n", f);
-  exit (1);
+  return -1;
 
 yuv_err:
   fprintf (stderr, "EOF parsing YUV4MPEG2 file %s.\n", f);
-  exit (1);
+  return -1;
 }
 
 int spinner = 0;
@@ -297,7 +288,7 @@ fetch_and_process_video (FILE *video, ogg_page *videopage, ogg_stream_state *to,
         if (memcmp (frame, "FRAME", 5))
         {
           fprintf (stderr, "Loss of framing in YUV input data\n");
-          exit (1);
+          exit (EXIT_FAILURE);
         }
         if (frame[5] != '\n')
         {
@@ -308,7 +299,7 @@ fetch_and_process_video (FILE *video, ogg_page *videopage, ogg_stream_state *to,
           if (j == 79)
           {
             fprintf (stderr, "Error parsing YUV frame header\n");
-            exit (1);
+            exit (EXIT_FAILURE);
           }
         }
 
@@ -354,7 +345,7 @@ fetch_and_process_video (FILE *video, ogg_page *videopage, ogg_stream_state *to,
          * can't get here unless YUV4MPEG stream has no video 
          */
         fprintf (stderr, "Video input contains no frames.\n");
-        exit (1);
+        exit (EXIT_FAILURE);
       }
 
       /*
@@ -434,48 +425,52 @@ main (int argc, char *argv[])
         if (outfile == NULL)
         {
           fprintf (stderr, "Unable to open output file '%s'\n", optarg);
-          exit (1);
+          return EXIT_FAILURE;
         }
         break;;
 
-      case 'v':
+      case 'q':
         video_q = rint (atof (optarg) * 6.3);
         if (video_q < 0 || video_q > 63)
         {
           fprintf (stderr, "Illegal video quality (choose 0 through 10)\n");
-          exit (1);
+          return EXIT_FAILURE;
         }
         video_r = 0;
         break;
 
-      case 'V':
+      case 'b':
         video_r = rint (atof (optarg) * 1000);
         if (video_r < 45000 || video_r > 2000000)
         {
           fprintf (stderr, "Illegal video bitrate (choose 45kbps through 2000kbps)\n");
-          exit (1);
+          return EXIT_FAILURE;
         }
         video_q = 0;
         break;
 
-      case 's':
-        video_an = rint (atof (optarg));
-        break;
-
-      case 'S':
-        video_ad = rint (atof (optarg));
+      case 'a':
+        if (sscanf (optarg, "%d/%d", &video_an, &video_ad) != 2)
+        {
+          fprintf (stderr, "Cannot parse aspect ratio\n");
+          return (EXIT_FAILURE);
+        }
         break;
 
       case 'f':
-        video_hzn = rint (atof (optarg));
+        if (sscanf (optarg, "%d/%d", &video_hzn, &video_hzd) != 2)
+        {
+          fprintf (stderr, "Cannot parse framerate\n");
+          return EXIT_FAILURE;
+        }
         break;
 
-      case 'F':
-        video_hzd = rint (atof (optarg));
-        break;
+      case 'h':
+        usage (argv[0]);
+        return EXIT_SUCCESS;
 
       default:
-        usage ();
+        return EXIT_FAILURE;
     }
   }
 
@@ -484,7 +479,8 @@ main (int argc, char *argv[])
     /*
      * assume that anything following the options must be a filename 
      */
-    id_file (argv[optind]);
+    if (id_file (argv[optind]) < 0)
+      return EXIT_FAILURE;
     optind++;
   }
 
@@ -510,8 +506,9 @@ main (int argc, char *argv[])
    */
   if (!video)
   {
-    usage ();
-    exit (1);
+    usage (argv[0]);
+
+    return EXIT_FAILURE;
   }
   /*
    * Theora has a divisible-by-sixteen restriction for the encoded video size 
@@ -570,7 +567,7 @@ main (int argc, char *argv[])
   if (ogg_stream_pageout (&to, &og) != 1)
   {
     fprintf (stderr, "Internal Ogg library error.\n");
-    exit (1);
+    return EXIT_FAILURE;
   }
   fwrite (og.header, 1, og.header_len, outfile);
   fwrite (og.body, 1, og.body_len, outfile);
@@ -598,7 +595,7 @@ main (int argc, char *argv[])
        * can't get here 
        */
       fprintf (stderr, "Internal Ogg library error.\n");
-      exit (1);
+      return EXIT_FAILURE;
     }
     if (result == 0)
       break;
@@ -670,6 +667,6 @@ main (int argc, char *argv[])
 
   fprintf (stderr, "\r   \ndone.\n\n");
 
-  return (0);
+  return EXIT_SUCCESS;
 }
 
